
# experimental function for getting basic KSSL data from CASRL
fetchKSSL <- function(series=NULL, bbox=NULL, mlra=NULL, pedlabsampnum=NULL, pedon_id=NULL, pedon_key=NULL, returnMorphologicData=FALSE, simplifyColors=FALSE) {
	
	# sanity-check: user must supply some kind of criteria
	if(missing(series) & missing(bbox) & missing(mlra) & missing(pedlabsampnum) & missing(pedon_id) & missing(pedon_key))
		stop('you must provide some filtering criteria')
	
	# init empty filter
	f <- vector()
	
	# init empty pieces
	s <- NULL
	h <- NULL
	
	# process filter components
	if(!missing(series)) {
		f <- c(f, paste('&series=', series, sep=''))
	}
	
	if(!missing(bbox)) {
		bbox <- paste(bbox, collapse=',')
		f <- c(f, paste('&bbox=', bbox, sep=''))
	}
	
	if(!missing(mlra)) {
	  f <- c(f, paste('&mlra=', mlra, sep=''))
	}
  
	if(!missing(pedlabsampnum)) {
	  f <- c(f, paste('&pedlabsampnum=', pedlabsampnum, sep=''))
	}
	
	if(!missing(pedon_id)) {
	  f <- c(f, paste('&pedon_id=', pedon_id, sep=''))
	}
	
	if(!missing(pedon_key)) {
	  f <- c(f, paste('&pedon_key=', pedon_key, sep=''))
	}
	
	# combine filters
	f <- paste(f, collapse='')
	
	# build URLs
	site.url <- URLencode(paste0('https://casoilresource.lawr.ucdavis.edu/soil_web/kssl/query.php?gzip=1&what=site', f))
	hz.url <- URLencode(paste0('https://casoilresource.lawr.ucdavis.edu/soil_web/kssl/query.php?gzip=1&what=horizon', f))
	morph.url <- URLencode(paste0('https://casoilresource.lawr.ucdavis.edu/soil_web/kssl/query.php?gzip=1&format=json&what=nasis_morphologic', f)) 
	
  # init temp files
	tf.site <- tempfile()
	tf.hz <- tempfile()
	
	# download pieces
	download.file(url=site.url, destfile=tf.site, mode='wb', quiet=TRUE)
	download.file(url=hz.url, destfile=tf.hz, mode='wb', quiet=TRUE)
  
  
	# load pieces
  # note use quote="" to ignore prime symbols (') in horizon names
	try(s <- read.table(gzfile(tf.site), header=TRUE, sep='|', stringsAsFactors=FALSE, quote='', comment.char=''), silent=TRUE)
	try(h <- read.table(gzfile(tf.hz), header=TRUE, sep='|', stringsAsFactors=FALSE, quote='', comment.char=''), silent=TRUE)
  
	# report missing data
	if(all(c(is.null(s), is.null(h)))) {
		message('query returned no data')
	  return(NULL)
	}
	
	
	# upgrade to SoilProfileCollection
	depths(h) <- pedon_key ~ hzn_top + hzn_bot
	site(h) <- s
	
	# 2016-04-22: basic morphologic data from NASIS
	if(returnMorphologicData) {
	  # check for required packages
	  if(!requireNamespace('jsonlite', quietly=TRUE))
	    stop('please install the `jsonlite` packages', call.=FALSE)
	  
	  
	  ## note: missing data are returned as FALSE
	  # get list of dataframe objects
	  m <- jsonlite::fromJSON(morph.url)
	  
	  ## TODO: consider simplification of fragment data here too
	  
	  # try to auto-simplify colors
	  if(simplifyColors & inherits(m$phcolor, 'data.frame')) {
	    # extract horizon data from SPC
	    hh <- horizons(h)
	    
	    # simplify color data: 1 row / horizon, from morphologic data tables
	    x.colors <- simplifyColorData(m$phcolor, id.var = 'labsampnum', wt='colorpct')
	    
	    # merge color data into SPC
	    hh <- join(hh, x.colors, by='labsampnum', type='left', match='first')
	    horizons(h) <- hh
	  }
	  
	} else m <- NULL
	
	# set KSSL-specific horizon identifier
	## WHOOPS -- turns out this is nonunique 0.1% of the time AGB 2019/11/14
	#hzidname(h) <- "labsampnum"
	
	# set KSSL-specific hzdesgn/hztexcl fields
	hzdesgnname(h) <- "hzn_desgn"
	hztexclname(h) <- "lab_texture_class"
	
	## set metadata
	h.metadata <- metadata(h)
	h.metadata$origin <- 'KSSL via Soilweb / fetchKSSL'
	metadata(h) <- h.metadata
	
	# report object size
	res.size <- round(object.size(h) / 1024 / 1024, 2) + round(object.size(m) / 1024 / 1024, 2)
	
	# some feedback via message:
	message(paste(length(h), ' pedons loaded (', res.size, ' Mb transferred)', sep=''))
  
	if(returnMorphologicData)
	  return(list(SPC=h, morph=m))
	else
	  return(h)
}
